/*
    libvice - SGI O2 Video Image Compression Engine Library
    Copyright (C) 2002 Vivien Chappelier
                       Ilya Volynets

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "vice.h"

vice_t *vice_init(int size, int verbose)
{
    vice_t *vice;
    int ret;
    unsigned long  rev;

    vice = (vice_t *) malloc(sizeof(vice_t));

    /* open device */
    if((vice->fd = open("/dev/vice", O_RDWR)) == -1) {
	fprintf(stderr, "error opening /dev/vice: %s\n", strerror(errno));
	return NULL;
    }

    /* map registers */
    if((vice->regs = 
        mmap(NULL, VICE_IO_MAX_OFFSET + 1,
             PROT_READ | PROT_WRITE,
             MAP_SHARED, vice->fd, 0)) == MAP_FAILED) {
	fprintf(stderr, "error mmapping I/O registers: %s\n", strerror(errno));
	return NULL;
    }
    
    /* map I/O buffer */
    vice->size = (size + ((1 << 16)-1)) & (~((1 << 16) - 1));
    if((vice->buffer = 
        mmap(NULL, vice->size,
             PROT_READ | PROT_WRITE,
             MAP_SHARED, vice->fd, VICE_MIN_OFFSET)) == MAP_FAILED){
        munmap(vice->regs, VICE_IO_MAX_OFFSET + 1);
	fprintf(stderr, "error mmapping I/O buffers: %s\n", strerror(errno));
	return NULL;
    }
    memset(vice->buffer, 0, vice->size);

    /* check revision */
    rev = vice_read_reg(vice, VICE_ID);
    rev = rev & 0xf;
    if(verbose)
        fprintf(stderr, "Detected VICE Revision %lx\n", rev);

    /* setup DMA TLB */
    if(ioctl(vice->fd, VICE_IOCTL_MAP_DMA, 0) == -1) {
	fprintf(stderr, "error mmapping buffer 0 into VICE: %s\n",
                strerror(errno));
	return NULL;
    }
    return vice;
}

void vice_cleanup(vice_t *vice)
{
    if(vice->regs != MAP_FAILED)
        munmap(vice->regs, VICE_IO_MAX_OFFSET + 1);
    if(vice->buffer != MAP_FAILED)
        munmap(vice->buffer, vice->size);
    close(vice->fd);
    free(vice);
}

void vice_debug_dump_dram(vice_t *vice)
{
    u32 i = VICE_DRAM_A;
    while(i < VICE_DRAM_C + 0x800) {
	u32 data;
	data = vice_read_reg32(vice,i);
	if((i % 32)==0)
	    fprintf(stderr, "\n%x: ",i);
	fprintf(stderr, "%08x ", data);
	i += 4;
    }
}

void vice_fill_dram(vice_t *vice, u32 value)
{
    u32 i = VICE_DRAM_A;
    while(i < VICE_DRAM_C + 0x800) {
	vice_write_reg32(vice, i, value);
	i += 4;
    }
}

int vice_load_data(vice_t *vice, int offset, void *data, int size)
{
    int i;

    u32 start = VICE_DRAM_A + offset;
    u32 end = VICE_DRAM_A + offset + size;
    if(end >= VICE_DRAM_C + 0x800) {
      fprintf(stderr, "data doesn't fit in VICE memory\n");
      return(0);
    }
    for(i = start; i < end; i += 4)
	vice_write_reg32(vice, i, ((u32 *) data)[i>>2]);
}

